#pragma once
#include <iostream>
#include<sstream>
#include "library/summation.hpp"
#include "library/stringstreamenumerator.hpp"
#include "library/maxsearch.hpp"
#include "library/enumerator.hpp"
#include "library/seqinfileenumerator.hpp"
#include "library/linsearch.hpp"

//#define TESZT

//A sorban ismtld adat szerkezete
struct adat{
    bool jo;
    std::string csat;
    int perc;
    friend std::istream& operator>>(std::istream& inp, adat& e){
        inp>>e.csat>>e.perc;
        e.jo=!inp.fail();
        return inp;
    }
};

//Ismtld adatok feldolgozsnak eredmnye
struct eredmeny{
    bool mind;
    int sum;
    eredmeny(){}
    eredmeny(bool log, int i):mind(log),sum(i){}
};

//Egy sorban az ismtld adatok feldolgozsa
class readLine: public Summation<adat,eredmeny>
{
protected:
    eredmeny func(const adat& e) const override {eredmeny x(e.perc>=60,e.perc); return x;}
    eredmeny neutral() const override {eredmeny x(true,0); return x;}
    eredmeny add(const eredmeny& a, const eredmeny& b) const override {eredmeny x(a.mind && b.mind, a.sum+b.sum); return x;}
    bool whileCond(const adat& e)const override {return e.jo;}
};


//Egy sorbl kinyert eredmny
struct sor{
    std::string id;
    std::string date;
    std::string kedvenc;
    int sum;
    bool mind;
    //Kir opertor, az  sszests ellenrzshez
    friend std::ostream& operator<<(std::ostream& out, const sor& e)
    {
        std::string log=e.mind? "igaz" : "hamis";
        out<<"Teszt: "<<e.id<<" "<<e.date<<" "<<e.kedvenc<<" "<<e.sum<<" "<<log<<"\n";
        return out;
    }
    friend std::istream& operator>>(std::istream& inp, sor& e)
    {
        std::string line;
        getline(inp,line);
        std::stringstream ss(line);
        ss>>e.id>>e.date;
        StringStreamEnumerator<adat> ssenor(ss);
        readLine pr;
        pr.addEnumerator(&ssenor);
        pr.run();
        e.mind=pr.result().mind;
        e.sum=pr.result().sum;
        e.kedvenc=ssenor.current().csat;
        return inp;
    }
};

//Megfelelt szint: feltteles maximum keress
class feltMax : public MaxSearch<sor, int>
{
protected:
    bool cond(const sor& e) const override {return e.mind && e.kedvenc=="NG";}
#ifdef TESZT
    int func(const sor& e) const override {std::cout<<e; return e.sum;}
#else
    int func(const sor& e) const override {return e.sum;}
#endif
};


//Kivalo szint egyedi felsorolo
//Sorok sszestsbl kapott sszegzs tpusa
struct efadat{
    std::string id;
    bool mind600;
    bool voltNG;
    //Kir opertor, az sszests ellenrzsre
    friend std::ostream& operator<<(std::ostream& out, const efadat& e)
    {std::string s1=e.mind600? "mind600": "nem mind600";
     std::string s2= e.voltNG? "voltNG" : "nemvoltNG";
     out<<"Teszt: "<<e.id<<" "<<s1<<" "<<s2<<std::endl; return out;}
    efadat() {}
    efadat(std::string s,bool l1, bool l2):id(s),mind600(l1),voltNG(l2){}

};

//Egy nz sorait sszest ttel
class nezoSum : public Summation<sor, efadat>
{
public:
    nezoSum(const std::string& nezoID): _aktID(nezoID){}
protected:
    efadat func(const sor& e) const override {efadat x(e.id, e.sum>=600,e.kedvenc=="NG"); return x;}
    efadat neutral() const override {efadat x(_aktID,true,false); return x;}
    efadat add(const efadat& a, const efadat&b) const override {efadat x(a.id, a.mind600 && b.mind600, a.voltNG || b.voltNG); return x;}
    void first() override {}
    bool whileCond(const sor& e) const override {return e.id==_aktID;}
    std::string _aktID;
};

//Egyedi felsorol, sszesti az azonos id-j nzket
class sajatEnumerator: public Enumerator<efadat>
{
private:
    SeqInFileEnumerator<sor>* _f;
    efadat _efadat;
    bool _end;
    //bool _empty; //ha az res felsorol gondot okozna
public:
    sajatEnumerator(const std::string& file) { _f = new SeqInFileEnumerator<sor>(file); }
    ~sajatEnumerator() { delete _f; }
    void first() override { _f->first(); next();}
    void next() override;
    efadat current() const override { return _efadat; }
    bool end() const override { return _end; }
};

void sajatEnumerator::next()
{
    _end=_f->end();
    if (_end) return;
    _efadat.id = _f->current().id;
    nezoSum pr(_efadat.id);
    pr.addEnumerator(_f);
    pr.run();
    _efadat = pr.result();
}

//Kivl szint progttel: lin.ker
class linKer: public LinSearch<efadat>
{
protected:
#ifdef TESZT
    bool cond(const efadat& e) const override {std::cout<<e; return e.mind600 && e.voltNG;}
#else
    bool cond(const efadat& e) const override {return e.mind600 && e.voltNG;}
#endif
};
